/*
 *  cannadic.c
 *  Copyright(C) 2003- Masahito Omote <omote@utyuuzin.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
 
#include "cannadic.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

void cannadic_parse_line(unsigned char *buf, word **head) {
    /*
     * cannadic format:
     *
     * phon cclass desc (desc desc ... ) (cclass desc (desc ...) ....)
     *
     * phon  : must
     * cclass: at least 1 entry
     * desc  : at least 1 entry
     *
     * cclass and desc must be associated with each other.
     *
     */
    unsigned char phon[1024], desc[1024], cclass_code[1024];
    int buflen;
    int desc_flag = 0, cclass_flag = 1;
    int i, j;

    if(buf[0] == '\0' || buf[0] == '#')
	return;

    buflen = strlen(buf);

    /* ɤߤ򸡽 */
    for(i = 0; i < buflen; i++) {
	if(buf[i] == ' ') {
	    phon[i] = '\0';
	    i++;
	    break;
	}
	phon[i] = buf[i];
    }

    for(j = 0 ; i <= buflen; i++, j++) {
	if(buf[i] == '#') {
	    cclass_flag = 1; desc_flag = 0;
	}

	if(buf[i] == ' '|| buf[i] == '\0') {
	    if(cclass_flag == 0 && desc_flag == 1) {
		desc[j] = '\0';
		if(buf[i+1] == '#') {
		    desc_flag = 0;
		    cclass_flag = 1;
		}
		word_append(head, WORD_TYPE_CANNA,
			    phon, desc, cclass_code,
			    0, 0, NULL);
	    } else if(cclass_flag == 1 && desc_flag == 0) {
		desc_flag = 1;
		cclass_flag = 0;
		cclass_code[j] = '\0';
	    }
	    j = -1;
	} else {
	    if(cclass_flag == 1 && desc_flag == 0) {
		cclass_code[j] = buf[i];
	    } else if (cclass_flag == 0 && desc_flag == 1) {
		desc[j] = buf[i];
	    }
	}
    }
}

GList *cannadic_parse_line_glist(unsigned char *buf, GList *list) {
    /*
     * cannadic format:
     *
     * phon cclass desc (desc desc ... ) (cclass desc (desc ...) ....)
     *
     * phon  : must
     * cclass: at least 1 entry
     * desc  : at least 1 entry
     *
     * cclass and desc must be associated with each other.
     *
     */
    unsigned char phon[1024], desc[1024], cclass_code[1024];
    gchar euc_literal[1024], euc_phonetic[1024], euc_pos[1024];
    gchar *literal, *phonetic, *pos;
    gchar cclass[1024], score[1024];
    gint ret = 0, rbytes, wbytes;

    int buflen;
    int desc_flag = 0, cclass_flag = 1;
    int i, j;

    if(buf[0] == '\0' || buf[0] == '#')
	return;

    buflen = strlen(buf);

    /* ɤߤ򸡽 */
    for(i = 0; i < buflen; i++) {
	if(buf[i] == ' ') {
	    phon[i] = '\0';
	    i++;
	    break;
	}
	phon[i] = buf[i];
    }

    for(j = 0 ; i <= buflen; i++, j++) {
	if(buf[i] == '#') {
	    cclass_flag = 1; desc_flag = 0;
	}

	if(buf[i] == ' '|| buf[i] == '\0') {
	    if(cclass_flag == 0 && desc_flag == 1) {
		desc[j] = '\0';
		if(buf[i+1] == '#') {
		    desc_flag = 0;
		    cclass_flag = 1;
		}
		phonetic = g_convert(phon, -1, "UTF-8", "EUC-JP", &rbytes, &wbytes, NULL);
		literal = g_convert(desc, -1, "UTF-8", "EUC-JP", &rbytes, &wbytes, NULL);
		pos = g_convert(cclass_code, -1, "UTF-8", "EUC-JP", &rbytes, &wbytes, NULL);
		score[0] = '\0'; /* XXX */
		cclass[0] = '\0'; /* XXX */
		list = dixchange_append_word(list, literal, phonetic, pos, cclass, score);
		g_free(phonetic);
		g_free(literal);
		g_free(pos);
	    } else if(cclass_flag == 1 && desc_flag == 0) {
		desc_flag = 1;
		cclass_flag = 0;
		cclass_code[j] = '\0';
	    }
	    j = -1;
	} else {
	    if(cclass_flag == 1 && desc_flag == 0) {
		cclass_code[j] = buf[i];
	    } else if (cclass_flag == 0 && desc_flag == 1) {
		desc[j] = buf[i];
	    }
	}
    }
    return list;
}

int cannadic_import(const char* filename, int imtype)
{
    FILE *fp;
    char buf[1024];
    int len;
    word *list = NULL, *pos = NULL;
	
    if(filename != NULL && filename[0] != '\0') {
	fp = fopen(filename, "r");
	if(!fp) {
	    return;
	}
	while(fgets(buf, sizeof(buf), fp)) {
	    len = strlen(buf);
	    buf[len-1] = '\0';
	    cannadic_parse_line(buf, &list);
	}
	for(pos = list; pos != NULL; pos = pos->next) {
	    switch (imtype) {
	    case 0: /* Anthy */
		add_anthy_priv_dic_with_flags(pos->phon,
					      pos->desc,
					      pos->cclass_code, 1);
		break;
	    case 1: /* Canna */
		snprintf(buf, sizeof(buf), "%s %s %s", pos->phon, pos->cclass_code, pos->desc);
		canna_define_dic("user", buf);
		break;
	    case 2: /* PRIME */
	    case 3: /* SKK */
	    default:
		break;
	    }
	}
	fclose(fp);
    }
}

char *find_pos_from_code(const char *code, int type) {
    /* need to be more smart */
    int i = 0, j = 0;
    char *pos = NULL;
    category_code *category[] = {
	substantive_code,
	verb_code,
	adjective_code,
	adverb_code,
	etc_code,
	NULL,
    };
    int num[] = { /* XXX */
	sizeof(substantive_code) / sizeof(substantive_code[0]),
	sizeof(verb_code) / sizeof(verb_code[0]),
	sizeof(adjective_code) / sizeof(adjective_code[0]),
	sizeof(adverb_code) / sizeof(adverb_code[0]),
	sizeof(etc_code) / sizeof(etc_code[0]),
	0,
    };

    do {
	for(j = 0; j < num[i]; j++) { /* XXX */
	    if(strcmp(code, (category[i])[j].code) == 0
	       && (category[i])[j].type & type == type)
		pos = strdup((category[i])[j].desc);
	}
	i++;
    } while(category[i] != NULL);

    return pos;
}

char *find_code_from_pos(const char *pos, int type) {
    /* need to be more smart */
    int i = 0, j = 0;
    char *code = NULL;
    category_code *category[] = {
	substantive_code,
	verb_code,
	adjective_code,
	adverb_code,
	etc_code,
	NULL,
    };
    int num[] = { /* XXX */
	sizeof(substantive_code) / sizeof(substantive_code[0]),
	sizeof(verb_code) / sizeof(verb_code[0]),
	sizeof(adjective_code) / sizeof(adjective_code[0]),
	sizeof(adverb_code) / sizeof(adverb_code[0]),
	sizeof(etc_code) / sizeof(etc_code[0]),
	0,
    };

    do {
	for(j = 0; j < num[i]; j++) { /* XXX */
	    if(strcmp(pos, (category[i])[j].desc) == 0
	       && (category[i])[j].type & type == type)
		code = strdup((category[i])[j].code);
	}
	i++;
    } while(category[i] != NULL);

    return code;
}
